@@ -25,6 +25,7 @@
 
 #include "common/debug_sysflags.h"
 #include "firebuild/exe_matcher.h"
+#include "firebuild/options.h"
 #include "firebuild/process.h"
 
 namespace firebuild {
@@ -37,6 +38,12 @@ void fb_error(const std::string &msg) {
   fprintf(stderr, "FIREBUILD ERROR: %s\n", msg.c_str());
 }
 
+void fb_info(const std::string &msg) {
+  if (!Options::quiet()) {
+    fprintf(stdout, "FIREBUILD: %s\n", msg.c_str());
+  }
+}
+
 void fb_debug(const std::string &msg) {
   if (!debug_suppressed) {
     fprintf(stderr, "FIREBUILD: %s\n", msg.c_str());
@@ -105,7 +112,7 @@ std::string pretty_timestamp() {
 #ifdef __APPLE__
   snprintf(buf, sizeof(buf), "%d-%02d-%02d %02d:%02d:%02d.%06d %c%02d%02d",
 #else
-  snprintf(buf, sizeof(buf), "%d-%02d-%02d %02d:%02d:%02d.%06ld %c%02d%02d",
+  snprintf(buf, sizeof(buf), "%d-%02d-%02d %02d:%02d:%02d.%06" PRItime " %c%02d%02d",
 #endif
       1900 + local.tm_year, 1 + local.tm_mon, local.tm_mday,
       local.tm_hour, local.tm_min, local.tm_sec, tv.tv_usec,
