@@ -20,8 +20,10 @@
 
 #include "firebuild/debug.h"
 #include "firebuild/blob_cache.h"
+#include "firebuild/config.h"
 #include "firebuild/file_info.h"
 #include "firebuild/file_name.h"
+#include "firebuild/utils.h"
 
 namespace firebuild {
 
@@ -76,6 +78,8 @@ static bool update_statinfo(const FileName* path, int fd, const struct stat64 *s
   entry->mtime = st->st_mtim;
   entry->inode = st->st_ino;
   entry->is_stored = false;
+  entry->is_static = false;
+  entry->is_static_checked = false;
   if (S_ISREG(st->st_mode)) {
     entry->info.set_type(ISREG);
     entry->info.set_mode_bits(st->st_mode & 07777, 07777 /* we know all the mode bits */);
@@ -297,11 +301,43 @@ bool HashCache::get_hash(const FileName* path, int max_writers, Hash *hash, bool
   return true;
 }
 
+#ifndef __APPLE__
+bool HashCache::get_is_static(const FileName* path, bool *is_static) {
+  TRACK(FB_DEBUG_HASH, "path=%s",
+      D(path));
+
+  if (!path || path->is_in_ignore_location()) {
+    return false;
+  }
+  const HashCacheEntry *entry = get_entry_with_statinfo(path, -1, nullptr);
+  if (entry->info.type() == NOTEXIST || entry->info.type() == DONTKNOW) {
+    return false;
+  }
+
+  if (!entry->is_static_checked) {
+    bool static_linked = is_statically_linked(path->c_str());
+    const_cast<HashCacheEntry*>(entry)->is_static = static_linked;
+    const_cast<HashCacheEntry*>(entry)->is_static_checked = true;
+  }
+
+  *is_static = entry->is_static;
+  return true;
+}
+#endif
+
 bool HashCache::store_and_get_hash(const FileName* path, int max_writers, Hash *hash,
-                                   off_t* stored_bytes, int fd, const struct stat64 *stat_ptr) {
+                                   off_t* stored_bytes, int fd, const struct stat64 *stat_ptr,
+                                   char **inline_data, size_t *inline_data_len) {
   TRACK(FB_DEBUG_HASH, "path=%s, max_writers=%d, fd=%d, stat=%s",
       D(path), max_writers, fd, D(stat_ptr));
 
+  if (inline_data) {
+    *inline_data = nullptr;
+  }
+  if (inline_data_len) {
+    *inline_data_len = 0;
+  }
+
   if (path->is_in_ignore_location()) {
     return false;
   }
@@ -311,6 +347,52 @@ bool HashCache::store_and_get_hash(const FileName* path, int max_writers, Hash *
     return false;
   }
   *hash = entry->info.hash();
+
+  /* If the file is small enough and caller requested inline data, read it into memory */
+  if (inline_data && inline_data_len && entry->info.type() == ISREG
+      && entry->info.size() > 0 && entry->info.size() <= max_inline_blob_size) {
+    char *buffer = static_cast<char*>(malloc(entry->info.size()));
+    if (buffer) {
+      int read_fd = fd;
+      bool need_close = false;
+      if (read_fd < 0) {
+        read_fd = open(path->c_str(), O_RDONLY);
+        need_close = true;
+      }
+      if (read_fd >= 0) {
+        off_t current_pos = -1;
+        if (fd >= 0) {
+          /* Save and restore file position if using provided fd */
+          current_pos = lseek(read_fd, 0, SEEK_CUR);
+          lseek(read_fd, 0, SEEK_SET);
+        }
+        ssize_t total_read = 0;
+        while (total_read < entry->info.size()) {
+          ssize_t n = read(read_fd, buffer + total_read, entry->info.size() - total_read);
+          if (n <= 0) {
+            break;
+          }
+          total_read += n;
+        }
+        if (total_read == entry->info.size()) {
+          *inline_data = buffer;
+          *inline_data_len = entry->info.size();
+          FB_DEBUG(FB_DEBUG_CACHING, "Inlined file data: " + d(path) + " size=" + d(total_read));
+        } else {
+          free(buffer);
+        }
+        if (current_pos >= 0) {
+          lseek(read_fd, current_pos, SEEK_SET);
+        }
+        if (need_close) {
+          close(read_fd);
+        }
+      } else {
+        free(buffer);
+      }
+    }
+  }
+
   return true;
 }
 
@@ -393,6 +475,79 @@ bool HashCache::file_info_matches(const FileName *path, const FileInfo& query) {
   return entry->info.hash() == query.hash();
 }
 
+const FileName* HashCache::resolve_command(const char* cmd, size_t cmd_len,
+                                          const char* path, size_t path_len, const FileName* wd) {
+  TRACK(FB_DEBUG_PROC, "cmd=%s, path=%s", D(cmd), D(path));
+
+  size_t candidate_buf_size = path_len + 1 + cmd_len + 1;
+  char* candidate_buf = static_cast<char*>(alloca(candidate_buf_size));
+  size_t candidate_len = 0;
+  assert(!path_is_absolute(cmd));
+  if (cmd_len == 0 || !path) {
+    /* Not a command to resolve on PATH or PATH is null. */
+    return nullptr;
+  }
+
+  const char *cursor = path;
+  while (true) {
+    const char *colon = strchr(cursor, ':');
+    if (colon == nullptr) {
+      candidate_len = path_len - (cursor - path);
+    } else {
+      candidate_len = colon - cursor;
+    }
+    memcpy(candidate_buf, cursor, candidate_len);
+    candidate_buf[candidate_len] = '\0';
+    // TODO(rbalint) this is not very efficient, consider caching canonical paths
+    // luckily this is not called that often
+    if (!is_canonical(candidate_buf, candidate_len)) {
+      size_t new_len = make_canonical(candidate_buf, candidate_len);
+      candidate_len = new_len;
+    }
+    candidate_buf[candidate_len] = '/';
+    memcpy(candidate_buf + candidate_len + 1, cmd, cmd_len);
+    candidate_buf[candidate_len + 1 + cmd_len] = '\0';
+    candidate_len += 1 + cmd_len;
+
+    if (!path_is_absolute(candidate_buf)) {
+      /* Make it absolute based on the working directory. */
+      if (!wd) {
+        /* Cannot resolve relative path without a working directory. */
+        if (colon == nullptr) {
+          break;
+        } else {
+          cursor = colon + 1;
+          continue;
+        }
+      }
+      const size_t wd_len = wd->length();
+      const size_t new_candidate_len = wd_len + 1 + candidate_len;
+      if (new_candidate_len + 1 > candidate_buf_size) {
+        char* candidate_buf2 = static_cast<char*>(alloca(new_candidate_len + 1));
+        memcpy(candidate_buf2, candidate_buf, candidate_len + 1);
+        candidate_buf = candidate_buf2;
+        /* candidate_buf_size is not valid anymore, but not used either. */
+      }
+      memmove(candidate_buf + wd_len + 1, candidate_buf, candidate_len + 1);
+      memcpy(candidate_buf, wd->c_str(), wd_len);
+      candidate_buf[wd_len] = '/';
+      candidate_len += wd_len + 1;
+    }
+    const FileName* candidate = FileName::Get(candidate_buf, candidate_len);
+    bool is_directory = false;
+    if (this->get_statinfo(candidate, &is_directory, nullptr)) {
+      if (!is_directory) {
+        return candidate;
+      }
+    }
+    if (colon == nullptr) {
+      break;
+    }
+    cursor = colon + 1;
+  }
+  return nullptr;
+}
+
 const HashCacheEntry HashCache::notexist_ {FileInfo(NOTEXIST)};
 const HashCacheEntry HashCache::dontknow_ {FileInfo(DONTKNOW)};
 
