@@ -38,6 +38,8 @@ struct HashCacheEntry {
   struct timespec mtime {};
   ino_t inode {};  /* skip device, it's unlikely to change */
   bool is_stored {};  /* it's known to be present in the blob cache because we stored it earlier */
+  bool is_static {}; /* it's a static binary detected to be run via qemu-user */
+  bool is_static_checked {}; /* whether we checked if it's a static binary */
 };
 
 /**
@@ -104,10 +106,13 @@ class HashCache {
    * @param[out] stored_bytes bytes stored to the blob cache
    * @param fd                if >= 0 then read the file from there
    * @param stat_ptr          optionally the file's parameters already stat()'ed
+   * @param[out] inline_data  if file is small enough, pointer to inline data (caller must free)
+   * @param[out] inline_data_len length of inline data if returned
    * @return                  false if not a regular file or directory
    */
   bool store_and_get_hash(const FileName* path, int max_writers, Hash *hash, off_t* stored_bytes,
-                          int fd, const struct stat64 *stat_ptr);
+                          int fd, const struct stat64 *stat_ptr,
+                          char **inline_data = nullptr, size_t *inline_data_len = nullptr);
 
   /**
    * Check if the given FileInfo query matches the file system.
@@ -118,6 +123,15 @@ class HashCache {
    */
   bool file_info_matches(const FileName *path, const FileInfo& query);
 
+  /** Resolve a command on the PATH */
+  const FileName* resolve_command(const char* cmd, size_t cmd_len,
+                                  const char* path, size_t path_len, const FileName* cwd);
+
+#ifndef __APPLE__
+  /** Check if the file is static */
+  bool get_is_static(const FileName* path, bool *is_static);
+#endif
+
  private:
   tsl::hopscotch_map<const FileName*, HashCacheEntry> db_ = {};
 
