@@ -31,15 +31,19 @@
 #include <utility>
 #include <vector>
 
+#include "common/config.h"
 #include "common/firebuild_common.h"
+#include "firebuild/command_rewriter.h"
 #include "firebuild/config.h"
 #include "firebuild/debug.h"
 #include "firebuild/connection_context.h"
+#include "firebuild/env.h"
 #include "firebuild/epoll.h"
 #include "firebuild/exe_matcher.h"
 #include "firebuild/file_name.h"
 #include "firebuild/execed_process.h"
 #include "firebuild/execed_process_cacher.h"
+#include "firebuild/hash_cache.h"
 #include "firebuild/pipe.h"
 #include "firebuild/pipe_recorder.h"
 #include "firebuild/process.h"
@@ -62,6 +66,21 @@ static void reject_exec_child(int fd_conn) {
     send_fbb(fd_conn, 0, reinterpret_cast<FBBCOMM_Builder *>(&sv_msg));
 }
 
+static void send_maybe_rewritten_cmd(int fd_conn, const FileName* executable,
+                                     std::vector<std::string>* args) {
+  FBBCOMM_Builder_rewritten_args sv_msg;
+  bool rewritten_executable = false;
+  bool rewritten_args = false;
+  CommandRewriter::maybe_rewrite(&executable, args, &rewritten_executable, &rewritten_args);
+  if (rewritten_executable) {
+    sv_msg.set_path(executable->c_str());
+  }
+  if (rewritten_args) {
+    sv_msg.set_arg(*args);
+  }
+  send_fbb(fd_conn, 0, reinterpret_cast<FBBCOMM_Builder *>(&sv_msg));
+}
+
 void MessageProcessor::accept_exec_child(ExecedProcess* proc, int fd_conn,
                                          int fd0_reopen) {
     TRACKX(FB_DEBUG_PROC, 1, 1, Process, proc, "fd_conn=%s, fd0_reopen=%s",
@@ -450,6 +469,11 @@ static void proc_new_process_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t
       assert_cmp(parent->state(), !=, FB_PROC_FINALIZED);
       if (parent->state() == FB_PROC_TERMINATED) {
         fds = parent->pass_on_fds();
+      } else if (parent->is_qemu()) {
+        /* This is a very special case. The parent is not terminated yet, but emulated an exec()
+         * of a dynamically linked binary inside the process. */
+        fds = parent->pass_on_fds();
+        // TODO(rbalint) maybe terminate parent?
       } else {
         /* Queue the ExecedProcess until parent's connection is closed */
         fds = new std::vector<std::shared_ptr<FileFD>>();
@@ -733,6 +757,26 @@ static void process_posix_spawn_file_actions(const P * const ic_msg, Process* co
 #endif
 }
 
+template<typename T>
+static const FileName* resolve_command_from_msg(const T* ic_msg, const char* command,
+                                                size_t command_len, Process* proc) {
+  if (ic_msg->get_with_p()) {
+    if (strchr(command, '/')) {
+      /* Resolve relative to current dir. */
+      return FileName::GetCanonicalized(command, command_len, proc->wd());
+    } else {
+      std::vector<std::string> env = ic_msg->get_env_as_vector();
+      size_t path_len = 0;
+      const char* path = Env::get_var(env, "PATH", &path_len);
+      return hash_cache->resolve_command(command, command_len, path, path_len,
+                                         proc->wd());
+    }
+  } else {
+    return proc->get_absolute(AT_FDCWD, command, command_len);
+  }
+}
+
+
 static void proc_ic_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t ack_num,
                         int fd_conn, Process* proc) {
   TRACKX(FB_DEBUG_COMM, 1, 1, Process, proc, "fd_conn=%s, tag=%s, ack_num=%d",
@@ -791,24 +835,25 @@ static void proc_ic_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t ack_num,
         break;
       }
 #endif
-      assert(proc->system_child());
+      Process* child = proc->system_child();
+      assert(child);
       /* system() implicitly waits for the child to finish. */
       int ret = ic_msg->get_ret();
       if (ret == -1 || !WIFEXITED(ret)) {
-        proc->system_child()->exec_point()->disable_shortcutting_bubble_up_to_excl(
-            proc->system_child()->fork_point()->exec_point(),
+        child->exec_point()->disable_shortcutting_bubble_up_to_excl(
+            child->fork_point()->exec_point(),
             "Process started by system() exited abnormally or the exit status could not be"
             " collected");
       } else {
-        proc->system_child()->fork_point()->set_exit_status(WEXITSTATUS(ret));
+        child->fork_point()->set_exit_status(WEXITSTATUS(ret));
       }
-      proc->system_child()->set_been_waited_for();
-      if (!proc->system_child()->fork_point()->can_ack_parent_wait()) {
-        /* The process has actually quit (otherwise the interceptor
-         * couldn't send us the system_ret message), but the supervisor
-         * hasn't seen this event yet. Thus we have to slightly defer
-         * sending the ACK. */
-        proc->system_child()->set_on_finalized_ack(ack_num, fd_conn);
+      child->set_been_waited_for();
+      if (child->exec_pending()) {
+        child->maybe_finalize_with_missed_static_exec_child();
+        /* Ack it straight away. */
+      } else if (!child->fork_point()->can_ack_parent_wait()) {
+        /* We haven't seen the process quitting yet. Defer sending the ACK. */
+        child->set_on_finalized_ack(ack_num, fd_conn);
         proc->set_system_child(NULL);
         return;
       }
@@ -879,7 +924,10 @@ static void proc_ic_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t ack_num,
           child->fork_point()->set_exit_status(WEXITSTATUS(ret));
         }
         child->set_been_waited_for();
-        if (!child->fork_point()->can_ack_parent_wait()) {
+        if (child->exec_pending()) {
+          child->maybe_finalize_with_missed_static_exec_child();
+          /* Ack it straight away. */
+        } else if (!child->fork_point()->can_ack_parent_wait()) {
           /* We haven't seen the process quitting yet. Defer sending the ACK. */
           child->set_on_finalized_ack(ack_num, fd_conn);
           return;
@@ -908,7 +956,12 @@ static void proc_ic_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t ack_num,
       /* The actual forked process might perform some file operations according to
        * posix_spawn()'s file_actions. Pre-open the files to be written. */
       posix_spawn_preopen_files(ic_msg, proc);
-      break;
+
+      const FileName* executable = resolve_command_from_msg(ic_msg, ic_msg->get_file(),
+                                                            ic_msg->get_file_len(), proc);
+      std::vector<std::string> args = ic_msg->get_arg_as_vector();
+      send_maybe_rewritten_cmd(fd_conn, executable, &args);
+      return;
     }
     case FBBCOMM_TAG_posix_spawn_parent: {
       auto ic_msg = reinterpret_cast<const FBBCOMM_Serialized_posix_spawn_parent *>(fbbcomm_buf);
@@ -1004,16 +1057,7 @@ static void proc_ic_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t ack_num,
 
       child->set_been_waited_for();
       if (child->exec_pending()) {
-        /* If the supervisor believes an exec is pending in a child proces while the parent
-         * actually successfully waited for the child, it means that the child didn't sign in to
-         * the supervisor, presumably because it is statically linked. See #324 for details. */
-        child->exec_point()->disable_shortcutting_bubble_up(
-            "Process did not sign in to supervisor, perhaps statically linked or failed to link");
-        /* Need to also clear the exec_pending state for Process::any_child_not_finalized()
-         * and finalize this never-seen process. */
-        child->set_exec_pending(false);
-        child->reset_file_fd_pipe_refs();
-        child->maybe_finalize();
+        child->maybe_finalize_with_missed_static_exec_child();
         /* Ack it straight away. */
       } else if (!child->fork_point()->can_ack_parent_wait()) {
         /* We haven't seen the process quitting yet. Defer sending the ACK. */
@@ -1037,7 +1081,23 @@ static void proc_ic_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t ack_num,
       proc->update_rusage(ic_msg->get_utime_u(), ic_msg->get_stime_u());
       // FIXME(rbalint) save exec parameters
       proc->set_exec_pending(true);
-      break;
+      const FileName* executable = nullptr;
+      if (ic_msg->has_fd()) {
+        int fd = ic_msg->get_fd();
+        if (fd != -1) {
+          const FileFD* file_fd = proc->get_fd(fd);
+          if (file_fd) {
+            executable = file_fd->filename();
+          }
+        }
+        assert(executable);
+      } else {
+        executable = resolve_command_from_msg(ic_msg, ic_msg->get_file(), ic_msg->get_file_len(),
+                                              proc);
+      }
+      std::vector<std::string> args = ic_msg->get_arg_as_vector();
+      send_maybe_rewritten_cmd(fd_conn, executable, &args);
+      return;
     }
     case FBBCOMM_TAG_pre_open: {
       PFBBA_HANDLE(proc, pre_open, fbbcomm_buf);
@@ -1219,7 +1279,10 @@ static void proc_ic_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t ack_num,
       } else {
         ExecedProcess* next_exec_level;
         if (quirks & FB_QUIRK_LTO_WRAPPER && proc->exec_point()->args().size() > 0
-            && proc->exec_point()->args()[0] == "touch"
+            && (proc->exec_point()->args()[0] == "touch"
+                || (proc->exec_point()->executable()->without_dirs() == "coreutils"
+                    && proc->exec_point()->args().size() > 1
+                    && proc->exec_point()->args()[1] == "--coreutils-prog-shebang=touch"))
             && (next_exec_level = proc->parent_exec_point())  // sh
             && (next_exec_level = next_exec_level->parent_exec_point())  // make
             && (next_exec_level = next_exec_level->parent_exec_point())  // lto-wrapper
@@ -1261,10 +1324,22 @@ static void proc_ic_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t ack_num,
       PFBBA_HANDLE(proc, socketpair, fbbcomm_buf);
       break;
     }
+    case FBBCOMM_TAG_mkfifo: {
+      PFBBA_HANDLE(proc, mkfifo, fbbcomm_buf);
+      break;
+    }
     case FBBCOMM_TAG_connect: {
       PFBBA_HANDLE(proc, connect, fbbcomm_buf);
       break;
     }
+    case FBBCOMM_TAG_bind: {
+      auto msg = reinterpret_cast<const FBBCOMM_Serialized_bind *>(fbbcomm_buf);
+      if (!msg->has_error_no()) {
+        proc->exec_point()->disable_shortcutting_bubble_up(
+            "Successfully binding network sockets is not supported");
+      }
+      break;
+    }
     case FBBCOMM_TAG_statfs:
       ProcessFBBAdaptor::handle(
            proc, reinterpret_cast<const FBBCOMM_Serialized_statfs *>(fbbcomm_buf));
@@ -1285,6 +1360,16 @@ static void proc_ic_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t ack_num,
       /* Ignore __mac_syscall(). Having the tag defined allows easier debugging with -d comm. */
       // TODO(rbalint) check if any __mac_syscall() invocation could impact builds
       break;
+    case FBBCOMM_TAG_rlimit: {
+      auto msg = reinterpret_cast<const FBBCOMM_Serialized_rlimit *>(fbbcomm_buf);
+      if (msg->has_error_no()) {
+        proc->exec_point()->disable_shortcutting_bubble_up(
+            "Changing resource limits failed");
+      } else {
+        /* Ignore success */
+      }
+      break;
+    }
     case FBBCOMM_TAG_fb_debug:
     case FBBCOMM_TAG_fb_error:
     case FBBCOMM_TAG_fchownat:
@@ -1304,7 +1389,7 @@ static void proc_ic_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t ack_num,
       const int error = msg->get_error_no_with_fallback(0);
       proc->exec_point()->disable_shortcutting_bubble_up(
           deduplicated_string(
-              std::string(error == 0 ? "" : "failed") + msg->get_call() + " is not supported"
+              std::string(error == 0 ? "" : "failed ") + msg->get_call() + " is not supported"
               + (error == 0 ? "" : " (error: " + d(error) + ")")).c_str());
       break;
     }
