@@ -188,6 +188,11 @@ void Process::AddPopenedProcess(int fd, ExecedProcess *proc) {
   fd2popen_child_[fd] = proc;
 }
 
+bool Process::is_qemu() const {
+  const ExecedProcess* exec_point_cached = exec_point();
+  return exec_point_cached && exec_point_cached->executable()->without_dirs().starts_with("qemu-");
+}
+
 int Process::handle_pre_open(const int dirfd, const char * const ar_name, const size_t ar_len) {
   TRACKX(FB_DEBUG_PROC, 1, 1, Process, this,
          "dirfd=%d, ar_name=%s", dirfd, D(ar_name));
@@ -509,10 +514,16 @@ int Process::handle_truncate(const char * const ar_name, const size_t ar_len,
   /* truncate() always sends pre_open. */
   name->close_for_writing();
 
-  // FIXME Will be a bit tricky to implement shortcutting, see #637.
-  (void)length;
-  (void)error;
-  exec_point()->disable_shortcutting_bubble_up("truncate() is not supported");
+  /* Truncate is basically open(..., O_WRONLY | length == 0 ? O_TRUNC : 0) && ftruncate(). */
+  FileUsageUpdate update =
+       FileUsageUpdate::get_from_open_params(name, O_WRONLY | (length == 0 ? O_TRUNC : 0), 0,
+                                             error, false);
+  if (!exec_point()->register_file_usage_update(name, update)) {
+    exec_point()->disable_shortcutting_bubble_up("Could not register truncate() on", *name);
+    return -1;
+  }
+  /* ftruncate() doesn't change the file's existence or type, no need to register anything else. */
+
   return 0;
 }
 
@@ -1181,6 +1192,36 @@ void Process::handle_socketpair(const int domain, const int type, const int prot
   }
 }
 
+int  Process::handle_mkfifo(const char * pathname, const size_t length,
+                            const mode_t mode, const int error) {
+  TRACKX(FB_DEBUG_PROC, 1, 1, Process, this, "pathname=%s, mode=%d, error=%d",
+         D(pathname), mode, error);
+
+  const FileName* name = get_absolute(AT_FDCWD, pathname, length);
+  if (!name) {
+    exec_point()->disable_shortcutting_bubble_up("Could not find file to create with mkfifo()");
+    return -1;
+  }
+
+  FileUsageUpdate update =
+      FileUsageUpdate::get_from_open_params(name, O_RDWR | O_CREAT | O_EXCL, mode, error, true);
+  if (!exec_point()->register_file_usage_update(name, update)) {
+    exec_point()->disable_shortcutting_bubble_up("Could not register mkfifo() on", *name);
+    return -1;
+  }
+
+  if (quirks & FB_QUIRK_LTO_WRAPPER && exec_point()->args().size() > 0
+      && exec_point()->args()[0] == "make"
+      && parent_exec_point()
+      && parent_exec_point()->executable()->without_dirs() == "lto-wrapper" ) {
+    FB_DEBUG(FB_DEBUG_PROC,
+             "Allow shortcutting mkfifo() in lto-wrapper's make (lto-wrapper quirk)");
+    return 0;
+  }
+  exec_point()->disable_shortcutting_bubble_up("mkfifo() is not supported");
+  return 0;
+}
+
 void Process::handle_connect(const int sockfd, const char * const address, const size_t address_len,
                              const int error) {
   TRACKX(FB_DEBUG_PROC, 1, 1, Process, this, "sockfd=%d, address=%s, error=%d",
@@ -1975,6 +2016,19 @@ void Process::maybe_finalize() {
   }
 }
 
+void Process::maybe_finalize_with_missed_static_exec_child() {
+  /* If the supervisor believes an exec is pending in a child process while the parent
+   * actually successfully waited for the child, it means that the child didn't sign in to
+   * the supervisor, presumably because it is statically linked. See #324 for details. */
+  exec_point()->disable_shortcutting_bubble_up(
+      "Process did not sign in to supervisor, perhaps statically linked or failed to link");
+  /* Need to also clear the exec_pending state for Process::any_child_not_finalized()
+   * and finalize this never-seen process. */
+  set_exec_pending(false);
+  reset_file_fd_pipe_refs();
+  maybe_finalize();
+}
+
 void Process::finish() {
   TRACKX(FB_DEBUG_PROC, 1, 1, Process, this, "");
 
