@@ -113,6 +113,11 @@ class Process {
    * as a subreaper it does not set the been_waited_for_ flag thus for those processes this function
    * never returns true. */
   virtual bool been_waited_for() const = 0;
+  /* If the supervisor believes an exec is pending in a child process while the parent
+   * actually successfully waited for the child, it means that the child didn't sign in to
+   * the supervisor, presumably because it is statically linked. See #324 for details.
+   * Call only when the process exited but still has a pending exec child. */
+  void maybe_finalize_with_missed_static_exec_child();
   virtual void set_been_waited_for() = 0;
   /**
    * The process is either finalized or is terminated, but in case it is just terminated it has
@@ -227,12 +232,18 @@ class Process {
   }
 
   /**
-   * Return the resolved and canonicalized absolute pathname, based on the given dirfd directory
+   * Return the resolved absolute pathname, based on the given dirfd directory
    * (possibly AT_FDCWD for the current directory). Return nullptr if the path is relative and
    * dirfd is invalid.
+   * @param dirfd directory file descriptor
+   * @param name relative or absolute file name, must be canonical
+   * @param length length of name, or -1 for strlen(name)
    */
   const FileName* get_absolute(const int dirfd, const char * const name, ssize_t length) const;
 
+  /** This is a qemu-user process. */
+  bool is_qemu() const;
+
   /**
    * Handle preparation for file opening in the monitored process
    * @param dirfd the dirfd of openat(), or AT_FDCWD
@@ -490,6 +501,12 @@ class Process {
   void handle_socketpair(const int domain, const int type, const int protocol,
                          const int fd0, const int fd1, const int error);
 
+  /**
+   * Handle mkfifo() in the monitored process
+   */
+  int handle_mkfifo(const char * const pathname, const size_t pathname_len,
+                    const mode_t mode, const int error);
+
   /**
    * Handle connect() in the monitored process
    */
