@@ -36,6 +36,7 @@
 #include <sys/resource.h>
 #include <spawn.h>
 
+#include "common/config.h"
 #include "interceptor/env.h"
 #include "interceptor/ic_file_ops.h"
 #include "interceptor/interceptors.h"
@@ -486,118 +487,6 @@ int clone_trampoline(void *arg) {
   return(trampoline_arg->orig_fn(trampoline_arg->orig_arg));
 }
 
-size_t make_canonical(char *path, size_t original_length) {
-  char *src = path, *dst = path;  /* dst <= src all the time */
-  bool add_slash = true;
-
-  if (path[0] == '\0') return 0;
-
-  if (!(path[0] == '.' && path[1] == '/')) {
-    char *a = strstr(path, "//");
-    char *b = strstr(path, "/./");
-    if (a == NULL && b == NULL) {
-      /* This is the quick code path for most of the well-behaved paths:
-       * doesn't start with "./", doesn't contain "//" or "/./".
-       * If a path passes this check then the only thing that might need
-       * fixing is a trailing "/" or "/.". */
-      size_t len = original_length;
-      if (len >= 2 && path[len - 1] == '.' && path[len - 2] == '/') {
-        /* Strip the final "." if the path ends in "/.". */
-        len--;
-        path[len] = '\0';
-      }
-      if (len >= 2 && path[len - 1] == '/') {
-        /* Strip the final "/" if the path ends in "/" and that's not the entire path. */
-        len--;
-        path[len] = '\0';
-      }
-      /* The quick code path is done here. */
-      return len;
-    }
-    /* Does not start with "./", but contains at least a "//" or "/./".
-     * Everything is fine up to that point. Fast forward src and dst. */
-    if (a != NULL && b != NULL) {
-      src = dst = a < b ? a : b;
-    } else if (a != NULL) {
-      src = dst = a;
-    } else {
-      src = dst = b;
-    }
-  } else {
-    /* Starts with "./", needs fixing from the beginning. */
-    src++;
-    add_slash = false;  /* Don't add "/" to dst when skipping the first one(s) in src. */
-  }
-
-  while (src[0] != '\0') {
-    /* Skip through a possible run of slashes and non-initial "." components, e.g. "//././". */
-    if (src[0] == '/') {
-      while (src[0] == '/' || (src[0] == '.' && (src[1] == '/' || src[1] == '\0'))) src++;
-      if (add_slash) {
-        *dst++ = '/';
-      }
-    }
-    /* Handle a regular (not ".") component. */
-    while (src[0] != '/' && src[0] != '\0') {
-      *dst++ = *src++;
-    }
-    add_slash = true;
-  }
-
-  /* If got empty path then it should be a "." instead. */
-  if (dst == path) {
-    *dst++ = '.';
-  }
-  /* Strip trailing slash, except if the entire path is "/". */
-  if (dst > path + 1 && dst[-1] == '/') {
-    dst--;
-  }
-
-  *dst = '\0';
-  return dst - path;
-}
-
-#if 0  /* unittests for make_canonical() */
-
-/* Macro so that assert() reports useful line numbers. */
-#define test(A, B) { \
-  char *str = strdup(A); \
-  make_canonical(str, strlen(str)); \
-  if (strcmp(str, B)) { \
-    fprintf(stderr, "Error:  input: %s\n", A); \
-    fprintf(stderr, "     expected: %s\n", B); \
-    fprintf(stderr, "  got instead: %s\n", str); \
-  } \
-}
-
-int main() {
-  test("/", "/");
-  test("/etc/hosts", "/etc/hosts");
-  test("/usr/include/vte-2.91/vte/vteterminal.h", "/usr/include/vte-2.91/vte/vteterminal.h");
-  test("/usr/bin/", "/usr/bin");
-  test("/usr/bin/.", "/usr/bin");
-  test("/usr/./bin", "/usr/bin");
-  test("/./usr/bin", "/usr/bin");
-  test("//", "/");
-  test("", "");
-  test(".", ".");
-  test("/.", "/");
-  test("./", ".");
-  test("/./././", "/");
-  test("./././.", ".");
-  test("//foo//bar//", "/foo/bar");
-  test("/././foo/././bar/././", "/foo/bar");
-  test("///.//././/.///foo//.//bar//.", "/foo/bar");
-  test("////foo/../bar", "/foo/../bar");
-  test("/foo/bar/../../../../../", "/foo/bar/../../../../..");
-  test("/.foo/.bar/..quux", "/.foo/.bar/..quux");
-  test("foo", "foo");
-  test("foo/bar", "foo/bar");
-  test("././foo/./bar/./.", "foo/bar");
-}
-
-#endif  /* unittests for canonicalize_path() */
-
 /** Store from entries from environment variable. */
 static void store_entries(const char* env_var, cstring_view_array *entries,
                           char * entries_env_buf, size_t buffer_size) {
@@ -1232,56 +1121,15 @@ void fb_ic_init() {
 
   fb_send_msg(fb_sv_conn, &ic_msg, 0);
 
-  /* Read the scproc_resp message header. */
-  msg_header header;
-#ifndef NDEBUG
-  ssize_t ret =
-#endif
-      fb_read(fb_sv_conn, &header, sizeof(header));
-  assert(ret == sizeof(header));
-  assert(header.msg_size > 0);
+  FBBCOMM_READ_MSG_HEADER_AND_ALLOC_BODY(fb_sv_conn, header, sv_msg_generic);
   uint16_t fd_count = header.fd_count;
-
-  /* Read the scproc_resp message body.
-   *
-   * This message may have file descriptors attached as ancillary data. */
-  FBBCOMM_Serialized *sv_msg_generic = alloca(header.msg_size);
-
-  void *anc_buf = NULL;
-  size_t anc_buf_size = 0;
-  if (fd_count > 0) {
-    anc_buf_size = CMSG_SPACE(fd_count * sizeof(int));
-    anc_buf = alloca(anc_buf_size);
-    memset(anc_buf, 0, anc_buf_size);
-  }
-
-  struct iovec iov = { 0 };
-  iov.iov_base = sv_msg_generic;
-  iov.iov_len = header.msg_size;
-
-  struct msghdr msgh = { 0 };
-  msgh.msg_iov = &iov;
-  msgh.msg_iovlen = 1;
-  msgh.msg_control = anc_buf;
-  msgh.msg_controllen = anc_buf_size;
+  FBBCOMM_CREATE_RECVMSG_HEADER_VAR_FD_COUNT(msgh, header, sv_msg_generic, fd_count);
 
   /* This is the first message arriving on the socket, and it's reasonably small.
    * We can safely expect that the header and the payload are fully available (no short read).
    * However, a signal interrupt might occur. */
-#ifndef NDEBUG
-  ret =
-#endif
-      TEMP_FAILURE_RETRY(
-#if defined(_TIME_BITS) && (_TIME_BITS == 64)
-          get_ic_orig___recvmsg64()(
-#else
-          get_ic_orig_recvmsg()(
-#endif
-              fb_sv_conn, &msgh, 0));
-      assert(ret >= 0 && ret == (ssize_t)header.msg_size);
-  assert(fbbcomm_serialized_get_tag(sv_msg_generic) == FBBCOMM_TAG_scproc_resp);
+  FBBCOMM_RECVMSG(scproc_resp, sv_msg, sv_msg_generic, fb_sv_conn, msgh, 0);
 
-  FBBCOMM_Serialized_scproc_resp *sv_msg = (FBBCOMM_Serialized_scproc_resp *) sv_msg_generic;
   debug_flags = fbbcomm_serialized_scproc_resp_get_debug_flags_with_fallback(sv_msg, 0);
 
   /* we may return immediately if supervisor decides that way */
