@@ -269,20 +269,6 @@ extern void insert_end_marker(const char*);
  */
 extern int ic_pid;
 
-/**
- * Make the filename canonical in place.
- *
- * String operation only, does not look at the actual file system.
- * Removes double slashes, trailing slashes (except if the entire path is "/")
- * and "." components.
- * Preserves ".." components, since they might point elsewhere if a symlink led to
- * its containing directory.
- * See #401 for further details and gotchas.
- *
- * Returns the length of the canonicalized path.
- */
-size_t make_canonical(char *path, size_t original_length);
-
 #ifdef FB_EXTRA_DEBUG
 static inline bool ic_cwd_ok() {
   char buf[FB_PATH_BUFSIZE];
@@ -342,6 +328,74 @@ static inline bool ic_cwd_ok() {
 #define BUILDER_MAYBE_SET_ABSOLUTE_CANONICAL(msg, dirfd, field)   \
   BUILDER_SET_CANONICAL2(msg, field, (dirfd == AT_FDCWD));
 
+#define FBBCOMM_READ_MSG_HEADER_AND_ALLOC_BODY(conn, header, msg_body)  \
+  msg_header header;                                                    \
+  fb_read(conn, &header, sizeof(header));                               \
+  assert(header.msg_size > 0);                                          \
+  FBBCOMM_Serialized *msg_body = alloca(header.msg_size)
+
+#define FBBCOMM_CREATE_RECVMSG_HEADER_COMMON(recvmsg_hdr, sv_msg_hdr, sv_msg_buf, fd_cnt, \
+                                             ctrl_buf, ctrl_len)        \
+  struct iovec iov = { 0 };                                             \
+  iov.iov_base = sv_msg_buf;                                            \
+  iov.iov_len = sv_msg_hdr.msg_size;                                    \
+                                                                        \
+  struct msghdr recvmsg_hdr = { 0 };                                    \
+  recvmsg_hdr.msg_iov = &iov;                                           \
+  recvmsg_hdr.msg_iovlen = 1;                                           \
+  recvmsg_hdr.msg_control = ctrl_buf;                                   \
+  recvmsg_hdr.msg_controllen = ctrl_len
+
+#define FBBCOMM_CREATE_RECVMSG_HEADER(recvmsg_hdr, sv_msg_hdr, sv_msg_buf, fd_count) \
+  union {                                                               \
+    char buf[CMSG_SPACE(fd_count * sizeof(int))];                       \
+    struct cmsghdr align;                                               \
+  } u = {};                                                             \
+  FBBCOMM_CREATE_RECVMSG_HEADER_COMMON(recvmsg_hdr, sv_msg_hdr, sv_msg_buf, fd_count, u.buf, \
+                                       sizeof(u.buf))
+/** FBBCOMM_CREATE_RECVMSG_HEADER() variant, where fd_count is not known at compile time. */
+#define FBBCOMM_CREATE_RECVMSG_HEADER_VAR_FD_COUNT(recvmsg_hdr, sv_msg_hdr, sv_msg_buf, fd_count) \
+  void *anc_buf = NULL;                                                 \
+  size_t anc_buf_size = 0;                                              \
+  if (fd_count > 0) {                                                   \
+    anc_buf_size = CMSG_SPACE(fd_count * sizeof(int));                  \
+    anc_buf = alloca(anc_buf_size);                                     \
+  memset(anc_buf, 0, anc_buf_size);                                     \
+  }                                                                     \
+  FBBCOMM_CREATE_RECVMSG_HEADER_COMMON(recvmsg_hdr, sv_msg_hdr, sv_msg_buf, fd_count, anc_buf, \
+                                       anc_buf_size)
+
+#if defined(_TIME_BITS) && (_TIME_BITS == 64)
+#define IC_ORIG_RECVMSG get_ic_orig___recvmsg64()
+#else
+#define IC_ORIG_RECVMSG get_ic_orig_recvmsg()
+#endif
+
+#define FBBCOMM_RECVMSG(tag, sv_msg, sv_msg_buf, conn, recvmsg_hdr, flags) \
+  TEMP_FAILURE_RETRY(IC_ORIG_RECVMSG(conn, &recvmsg_hdr, flags)); \
+  assert(fbbcomm_serialized_get_tag((FBBCOMM_Serialized *) sv_msg_buf) == FBBCOMM_TAG_##tag); \
+  FBBCOMM_Serialized_##tag *sv_msg = (FBBCOMM_Serialized_##tag *) sv_msg_buf
+
+#define FBBCOMM_ALLOC_AND_RECVMSG(tag_name, sv_msg, conn)          \
+  FBBCOMM_READ_MSG_HEADER_AND_ALLOC_BODY(conn, sv_msg_hdr, sv_msg_buf); \
+  FBBCOMM_CREATE_RECVMSG_HEADER(msgh, sv_msg_hdr, sv_msg_buf, 0);       \
+  FBBCOMM_RECVMSG(tag_name, sv_msg, sv_msg_buf, conn, msgh, 0)
+
+#define FBBCOMM_ALLOC_AND_REWRITE_ARGS(sv_msg, new_argv, argv)               \
+  unsigned int new_argc = fbbcomm_serialized_rewritten_args_get_arg_count(sv_msg); \
+  if (new_argc > 0) {                                                   \
+    unsigned int i;                                                     \
+    /* Rebuild argv on the stack */                                     \
+    new_argv = alloca(sizeof(char *) * (new_argc + 1));                 \
+    for (i = 0; i < new_argc; i++) {                                    \
+      new_argv[i] = (char *) fbbcomm_serialized_rewritten_args_get_arg_at(sv_msg, i); \
+    }                                                                   \
+    new_argv[new_argc] = NULL;                                          \
+  } else {                                                              \
+    /* No change */                                                     \
+    new_argv = (char **)argv;                                           \
+  }
+
 typedef struct {
   /** The method name the current thread is intercepting, or NULL. In case of nested interceptions
    *  (which can happen with signal handlers), it contains the outermost intercepted method. The
@@ -395,7 +449,7 @@ bool signal_is_wrappable(int);
 /** When a signal handler is installed using signal(), sigset(), or sigaction() without the
  *  SA_SIGINFO flag, this wrapper gets installed instead.
  *
- *  See tpl_signal.c for how this wrapper is installed instead of the actual handler.
+ *  See tpl_signal.c.jinja2 for how this wrapper is installed instead of the actual handler.
  *
  *  This wrapper makes sure that the actual signal handler is only executed straight away if the
  *  thread is not inside a "signal danger zone". Otherwise execution is deferred until the danger
